package org.kaoriha.jspbp;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletResponse;

@SuppressWarnings("deprecation")
class DelegateResponse implements HttpServletResponse {
	private HttpServletResponse response;
	private boolean isRedirected = false;
	private boolean isError = false;
	private String requestURI;
	private String persistentId;
	private String continueId;
	private QueryString queryString = new QueryString();

	DelegateResponse(HttpServletResponse response, String requestURI) {
		this.response = response;
		this.requestURI = requestURI;
	}

	public boolean isRedirected() {
		return isRedirected;
	}

	public boolean isError() {
		return isError;
	}

	public void setPersistentId(String persistentId) {
		this.persistentId = persistentId;
	}

	public void setContinueId(String continueId) {
		this.continueId = continueId;
	}

	public void addCookie(Cookie cookie) {
		response.addCookie(cookie);
	}

	public void addDateHeader(String name, long date) {
		response.addDateHeader(name, date);
	}

	public void addHeader(String name, String value) {
		response.addHeader(name, value);
	}

	public void addIntHeader(String name, int value) {
		response.addIntHeader(name, value);
	}

	public boolean containsHeader(String name) {
		return response.containsHeader(name);
	}

	public String encodeRedirectUrl(String url) {
		return response.encodeRedirectUrl(url);
	}

	public String encodeRedirectURL(String url) {
		return response.encodeRedirectURL(url);
	}

	public String encodeUrl(String url) {
		return response.encodeUrl(url);
	}

	public String encodeURL(String url) {
		return response.encodeURL(url);
	}

	public void flushBuffer() throws IOException {
		response.flushBuffer();
	}

	public int getBufferSize() {
		return response.getBufferSize();
	}

	public String getCharacterEncoding() {
		return response.getCharacterEncoding();
	}

	public String getContentType() {
		return response.getContentType();
	}

	public Locale getLocale() {
		return response.getLocale();
	}

	public ServletOutputStream getOutputStream() throws IOException {
		return response.getOutputStream();
	}

	public PrintWriter getWriter() throws IOException {
		return response.getWriter();
	}

	public boolean isCommitted() {
		return response.isCommitted();
	}

	public void reset() {
		response.reset();
	}

	public void resetBuffer() {
		response.resetBuffer();
	}

	public void sendError(int sc, String msg) throws IOException {
		isError = true;
		response.sendError(sc, msg);
	}

	public void sendError(int sc) throws IOException {
		isError = true;
		response.sendError(sc);
	}

	public void sendRedirect(String location) throws IOException {
		isRedirected = true;
		String base = parseQuery(location);
		if (requestURI.equals(base) && persistentId != null) {
			List<String> sl = new ArrayList<String>();
			sl.add(persistentId);
			queryString.put(Request.PERSISTENT_ID_QN, persistentId);
		}
		if (continueId != null) {
			queryString.put(Request.CONTINUE_ID_QN, continueId);
		}
		response.sendRedirect(buildLocation(base));
	}

	private String parseQuery(String location) {
		int qi = location.indexOf('?');
		if (qi == -1) {
			return location;
		}
		String base = location.substring(0, qi);
		queryString = new QueryString(location.substring(qi + 1));
		return base;
	}

	private String buildLocation(String base) {
		return queryString.build(base);
	}

	public void setBufferSize(int size) {
		response.setBufferSize(size);
	}

	public void setCharacterEncoding(String charset) {
		response.setCharacterEncoding(charset);
	}

	public void setContentLength(int len) {
		response.setContentLength(len);
	}

	public void setContentType(String type) {
		response.setContentType(type);
	}

	public void setDateHeader(String name, long date) {
		response.setDateHeader(name, date);
	}

	public void setHeader(String name, String value) {
		response.setHeader(name, value);
	}

	public void setIntHeader(String name, int value) {
		response.setIntHeader(name, value);
	}

	public void setLocale(Locale loc) {
		response.setLocale(loc);
	}

	public void setStatus(int sc, String sm) {
		response.setStatus(sc, sm);
	}

	public void setStatus(int sc) {
		response.setStatus(sc);
	}

}
