package org.kaoriha.jspbp;

import java.io.IOException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * This class captures request for JSP. <br>
 * You must insert init-param element in web.xml. <br>
 * param-name:<code>behindPackageName</code>. The package name of behind bean classes.<br>
 * param-name:<code>encoding</code>. Specify your JSP's page encoding. Default:UTF-8.
 * 
 * @author NAKAZATO Hajime
 */
public class JspbpFilter implements Filter {
	private static final String IPK_PACKAGE_NAME = "behindPackageName";
	private String behindPackageName;

	private static final String IPK_ENCODING = "encoding";
	private String encoding = "UTF-8";

	private static final String IPK_TEMP_DIR = "tempDir";
	private String tempDir;

	private static final String IPK_MAX_FILESIZE = "maxFilesize";
	private long maxFilesize = 1024 * 1024 * 10;

	private static final String IPK_SIZE_EXCEED_PATH = "sizeExceedPath";
	private String sizeExceedPath;

	private ServletContext servletContext;

	public void destroy() {
	}

	public void doFilter(ServletRequest request, ServletResponse response,
			FilterChain chain) throws IOException, ServletException {
		doHttpFilter((HttpServletRequest) request,
				(HttpServletResponse) response, chain);
	}

	private void doHttpFilter(HttpServletRequest request,
			HttpServletResponse response, FilterChain chain)
			throws IOException, ServletException {
		request.setCharacterEncoding(encoding);
		request.getParameterMap(); // Don't remove. It fixes encoding.
		new Request(request, response, chain, this).doFilter();
	}

	public void init(FilterConfig filterConfig) throws ServletException {
		servletContext = filterConfig.getServletContext();

		behindPackageName = filterConfig.getInitParameter(IPK_PACKAGE_NAME);
		if (behindPackageName == null) {
			throw new ServletException(
					"You must insert init-param element in web.xml. param-name:"
							+ IPK_PACKAGE_NAME);
		}

		tempDir = filterConfig.getInitParameter(IPK_TEMP_DIR);

		String filesizeStr = filterConfig.getInitParameter(IPK_MAX_FILESIZE);
		if (filesizeStr != null) {
			try {
				maxFilesize = Long.parseLong(filesizeStr);
			} catch (NumberFormatException e) {
				throw new ServletException(
						"Invalid init-param element in web.xml. param-name:"
								+ IPK_MAX_FILESIZE);
			}
		}

		String enc = filterConfig.getInitParameter(IPK_ENCODING);
		if (enc != null) {
			encoding = enc;
		}

		sizeExceedPath = filterConfig.getInitParameter(IPK_SIZE_EXCEED_PATH);
	}

	String getBehindPackageName() {
		return behindPackageName;
	}

	String getTempDir() {
		return tempDir;
	}

	long getMaxFilesize() {
		return maxFilesize;
	}

	String getEncoding() {
		return encoding;
	}

	String getSizeExceedPath() {
		return sizeExceedPath;
	}

	ServletContext getServletContext() {
		return servletContext;
	}
}
